//Author: Ate Akaydn
//Date: 12.05.2008
//This class holds the constants used in the Vorgen project

#ifndef VG_CONSTANTS
#define VG_CONSTANTS

#include <map>
#include "vg_color.h"
#include <math.h>
#include <vector>
#include <iostream>
#include <boost/graph/astar_search.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/random.hpp>
#include <boost/random.hpp>
#include <boost/graph/graphviz.hpp>
#include <boost/graph/graph_utility.hpp>

#define VG_DEFAULT_POINTSIZE 1
#define VG_DEFAULT_NODESIZE 2
#define VG_DEFAULT_EDGESIZE 2
#define VG_DEFAULT_EDGEWEIGHT 999999
#define VG_DEFAULT_LINEWIDTH 1
#define VG_DEFAULT_PATHWIDTH 6
#define VG_DEFAULT_DECODER_SIZE 10000
#define VG_DEFAULT_WINDOW_SIZE 600
#define VG_DEFAULT_ERROR_THRESHOLD 1
#define VG_DRAW_SITES 1

//switch to namespace boost
using namespace boost;

class VG_Point2d;

//Agent structure
struct agent{

    double pos_x,pos_y;
    double dir_x,dir_y;
    double speed;
    VG_Point2d * site;

};

struct location{
    double x,y;
};

struct node{
    double x,y;
    bool sampleNode;
    std::vector<unsigned int> adj;
};

struct window{
    unsigned int row,col;
};

extern bool colorMapPrint;

struct colorCmp{

	bool operator()(const VG_Color & c1, const VG_Color & c2 ) const{


		unsigned int id1 = static_cast<unsigned int>( ( c1.r )*(256*256) + ( c1.g * 256 ) + c1.b );
		unsigned int id2 = static_cast<unsigned int>( ( c2.r )*(256*256) + ( c2.g * 256 ) + c2.b );


        bool ret = false;

		if( id1 > id2 )
            ret = true;

        if( colorMapPrint ){
            std::cout << "color1:" << c1.r << "," << c1.g << "," << c1.b << std::endl;
            std::cout << "color2:" << c2.r << "," << c2.g << "," << c2.b << std::endl;
            std::cout << "color id1:" << id1 << std::endl;
            std::cout << "color id2:" << id2 << std::endl;
            std::cout << "id1 < id2:" << (ret) << std::endl;
        }

        return ret;
	}
};

typedef std::map< VG_Color, unsigned int, colorCmp > ColorMap;
typedef ColorMap::iterator ColorItr;
typedef std::pair< VG_Color, unsigned int > ColorPair;

//Boost Graph definitions

//Adj list definition
typedef adjacency_list< listS, vecS, undirectedS, no_property,
    property< edge_weight_t, double > > vg_graph;

//Edge weight map definition
typedef property_map<vg_graph, edge_weight_t>::type vg_weightmap;

//Vertex & Edge descriptors
typedef vg_graph::vertex_descriptor vg_vertex;
typedef vg_graph::edge_descriptor vg_edge_desc;


//Vertex iterator
typedef vg_graph::vertex_iterator vg_vertex_itr;

//Euclidean distance heuristic
template <class Graph, class CostType, class LocMap>

//Distance heuristic
class distance_heuristic : public astar_heuristic<Graph, CostType>
{
public:
  typedef typename graph_traits<Graph>::vertex_descriptor Vertex;
  distance_heuristic(LocMap l, Vertex goal)
    : m_location(l), m_goal(goal) {}
  CostType operator()(Vertex u)
  {
    CostType dx = m_location[m_goal].x - m_location[u].x;
    CostType dy = m_location[m_goal].y - m_location[u].y;
    return ::sqrt(dx * dx + dy * dy);
  }
private:
  LocMap m_location;
  Vertex m_goal;
};

struct vg_graph_data{

    //Nodes and node locations
    location * locations;

    //Edges
    std::pair< unsigned int, unsigned int> * edges;

    //Edge weights
    double * weights;

    double maxWeight;

    double minWeight;

    //Number of seeds nodes within the graph
    unsigned int numSeed;

    //Number of nodes in the graph
    unsigned int numNodes;

    //Number of edges in the graph
    unsigned int numEdges;

    //Adjacency list
    vg_graph* adj_list;

};


struct found_goal {};

// visitor that terminates when we find the goal
template <class Vertex>
class astar_goal_visitor : public boost::default_astar_visitor
{
public:
  astar_goal_visitor(Vertex goal) : m_goal(goal) {}
  template <class Graph>
  void examine_vertex(Vertex u, Graph& g) {
    if(u == m_goal)
      throw found_goal();
  }
private:
  Vertex m_goal;
};


#endif
