//Autor: Ate Akaydn
//Date: 12.05.2008
/*
	This is the voronoi site color decoder implementation file. It basically decodes a given site id to some color
	Also encodes a 2d RGB color buffer into a 2d site id buffer

	Colors here are represented as normalized colors - where r,g,b values are in between [0,1]. Any given 8-bit(0-255) color
	format for bands are explicitly normalized within the relevant functions.
*/

#include "vg_decoder.h"

extern bool colorMapPrint = false;
//Default constructor of a decoder
//Generates given number of colors and finds their encoding
VG_Decoder::VG_Decoder( unsigned int nmColors ):numColors(nmColors){

    srand( time( NULL ) );

    colors = new VG_Color[numColors];

    unsigned int i = 0;

    while( i < numColors ){

        GLubyte r = static_cast<GLubyte>( ( ( ( double )( rand() ) )/RAND_MAX ) * 255 );
        GLubyte g = static_cast<GLubyte>( ( ( ( double )( rand() ) )/RAND_MAX ) * 255 );
        GLubyte b = static_cast<GLubyte>( ( ( ( double )( rand() ) )/RAND_MAX ) * 255 );

        VG_Color tmp_color = VG_Color( r, g, b );

        std::pair< ColorItr, bool > p = colorMap.insert( ColorPair( tmp_color,i ) );

        //If this new color is not inserted
        if( !p.second ){
            continue;
        }

        colors[i] = tmp_color;
        i++;


    }
}

//Destructor for decoder class
VG_Decoder::~VG_Decoder(){
    delete[] colors;
}

//Returns number of colors
unsigned int VG_Decoder::getNumColors() const{
    return numColors;
}

//Decodes a given index into its corresponding color and returns it
VG_Color VG_Decoder::decodeColor( unsigned int ind ) const{
    assert( ind < numColors && ind >= 0 );
    return colors[ind];
}

//Encodes a given color and returns its corresponding site id
//Exists variable determines whether a particular encoding is found
unsigned int VG_Decoder::encodeIndex( VG_Color c, bool & exists ) {
    //colorMapPrint = true;
    ColorItr itr = colorMap.find( c );
    //colorMapPrint = false;
    if( itr == colorMap.end() ){
        exists = false;
        return 0;
    }
    else{
        exists = true;
        return (*itr).second;
    }
}

//Encodes a given RGB color buffer into a buffer of site ids
unsigned int ** VG_Decoder::encodeRGBBuffer( GLubyte * buffer, unsigned int sx, unsigned int sy ){

    //Burda row onceligi taninabilinir
    unsigned int ** ret = new unsigned int *[sy];
    for( int y = 0; y < sy; y++ ){
        ret[y] = new unsigned int[sx];
        for( unsigned int x = 0; x < sx; x++ ){
            VG_Color tmpColor = VG_Color( static_cast<GLubyte>(buffer[static_cast<unsigned int>((sy-y-1)*sx*3 + x*3 + 0)]),
                                          static_cast<GLubyte>(buffer[static_cast<unsigned int>((sy-y-1)*sx*3 + x*3 + 1)]),
                                          static_cast<GLbyte>(buffer[static_cast<unsigned int>((sy-y-1)*sx*3 + x*3 + 2)]) );
            bool exists = false;
            ret[y][x] = encodeIndex( tmpColor,  exists ); //If not exists we should do something

        }
    }
    return ret;
}
