//Autor: Ate Akaydn
//Date: 12.05.2008
/*
	This is the voronoi site color decoder class. It basically decodes a given site id to some color
	Also encodes a 2d RGB color buffer into a 2d site id buffer

	Colors here are represented as normalized colors - where r,g,b values are in between [0,1]. Any given 8-bit(0-255) color
	format for bands are explicitly normalized within the relevant functions.
*/

#ifndef VG_DECODER_H
#define VG_DECODER_H

#include <windows.h>
#include <gl.h>
#include <glu.h>
//#include <gl/glaux.h>
#include <assert.h>
#include <time.h>
#include <cstdlib>
//#include <iostream>

#include "vg_color.h"
#include "vg_constants.h"

class VG_Decoder{

private:

	VG_Color * colors;
	ColorMap colorMap;
	unsigned int numColors;


public:

	//Default constructor of a decoder
	//Generates given number of colors and finds their encoding
	VG_Decoder( unsigned int nmColors );

	//Destructor for decoder class
	~VG_Decoder();

	//Returns number of colors
	unsigned int getNumColors() const;

	//Decodes a given index into its corresponding color and returns it
	VG_Color decodeColor( unsigned int ind ) const;

	//Encodes a given color and returns its corresponding site id
	//Exists variable determines whether a particular encoding is found
	unsigned int encodeIndex( VG_Color c, bool & exists );

	//Encodes a given RGB color buffer into a buffer of site ids
	unsigned int ** encodeRGBBuffer( GLubyte * buffer, unsigned int sx, unsigned int sy );

};

#endif
