//Autor: Ate Akaydn, Gnen Ercan
//Date: 27.05.2008
/*
    Grapher class to construct a graph from the given site buffer
*/

#include <iostream>
#include <stack>
#include <vector>
#include <list>
#include "vg_constants.h"

class VG_Graph{

private:

    unsigned int ** grid;
    unsigned int size_x,size_y;
    std::stack<window *> wstack;
    bool ** marked;
    std::vector<node *> graphnodes;
    std::vector<node *> seednodes;

public:


    VG_Graph(unsigned int **inGrid, unsigned int sizex, unsigned int sizey ):size_x(sizex),size_y(sizey),grid(inGrid) {
        graphnodes.clear();
        seednodes.clear();
        marked = NULL;
    }
    ~VG_Graph(){

    }
    void findGraphNodes() {


        //Safe coding
        delWStack();
        delMarked();
        delNodes();

        std::cout << "Graphing..." << std::endl;

        //Find seeds
        findSeeds();

        //Construct marked array
        buildMarked( );

        //Find nodes of the graph and construct the graph
        findNodes();

        printNodes();

        std::cout << "Graphing done." << std::endl;


    }
    void delWStack(){
            window * tmp;
            while( !wstack.empty() ){
                tmp = wstack.top();
                wstack.pop();
                delete tmp;
            }
    }
    void buildMarked(){

        if( marked != NULL ) return;
        //Boolean marked matrisini olustur
        marked = new bool *[size_y+1];
        for( int i = 0; i < size_y+1; i++ ){
            marked[i] = new bool[size_x+1];
            for( int j = 0; j < size_x+1; j++ ){
                if( i == 0 || i == size_y || j == 0 || j == size_x )
                    marked[i][j] = true;
                else
                    marked[i][j] = false;
            }
        }

    }
    void delMarked(){

        if( marked == NULL ) return;

        for( int i = 0; i < size_y+1; i++ ){
            delete[] marked[i];
        }
        delete[] marked;

        marked = NULL;

    }
    void mark( window * w ){
        marked[w->col+1][w->row+1] = true;
    }
    bool checkMarked( window * w ){
        return marked[w->col+1][w->row+1];
    }
    //Find seeds
    void findSeeds(){

        window * seed;

        // Move around top boundary
        for(int i=0; i<size_x-1; i++)
        {
            if( grid[0][i] != grid[0][i+1] ){
                    seed = new window();
                    seed->col = i;
                    seed->row = 0;
                    //std::cout << "Seed found at [" << seed->row << "," << seed->col << "]" << std::endl;
                    wstack.push( seed );
                    std::vector<unsigned int> unq = getUnique( seed );
                    node * snode = wToNode( seed, unq );
                    graphnodes.push_back( snode );
                    seednodes.push_back( snode );
            }
        }

        // Move around right boundary
        for(int i=0; i<size_y-1; i++)
        {
            if( grid[i][size_x-1] != grid[i+1][size_x-1] ){
                    seed = new window();
                    seed->col = size_x-2;
                    seed->row = i;
                    //std::cout << "Seed found at [" << seed->row << "," << seed->col << "]" << std::endl;
                    wstack.push( seed );

                    std::vector<unsigned int> unq = getUnique( seed );
                    node * snode = wToNode( seed, unq );
                    graphnodes.push_back( snode );
                    seednodes.push_back( snode );
            }
        }


        // Move around bottom boundary
        for( int i = 0; i < size_x-1; i++ )
        {
            if( grid[size_y-1][i] != grid[size_y-1][i+1] ){
                    seed = new window();
                    seed->col = i;
                    seed->row = size_y-2;
                    //std::cout << "Seed found at [" << seed->row << "," << seed->col << "]" << std::endl;
                    wstack.push( seed );

                    std::vector<unsigned int> unq = getUnique( seed );
                    node * snode = wToNode( seed, unq );
                    graphnodes.push_back( snode );
                    seednodes.push_back( snode );
            }
        }

        // Move around left boundary
        for(int i = 0; i < size_y-1; i++ )
        {
            if( grid[i][0] != grid[i+1][0] ){
                    seed = new window();
                    seed->col = 0;
                    seed->row = i;
                    //std::cout << "Seed found at [" << seed->row << "," << seed->col << "]" << std::endl;
                    wstack.push( seed );

                    std::vector<unsigned int> unq = getUnique( seed );
                    node * snode = wToNode( seed, unq );
                    graphnodes.push_back( snode );
                    seednodes.push_back( snode );
            }
        }

    }

    //Find nodes
    void findNodes(){
        window * cur_window = NULL;
        while( !wstack.empty() ){
            cur_window = wstack.top();
            wstack.pop();
            if( checkMarked( cur_window ) ) continue;

            mark(cur_window);

            std::vector<unsigned int> unq = getUnique( cur_window );
            switch( unq.size() ){
                case(1): //If this window is withing a site
                    continue;
                    break;
                case(2): //If this window is an edge
                    {
                        node * p = wToNode( cur_window, unq );
                        graphnodes.push_back( p );
                    }
                    break;
                default: //If this window is a node
                    {
                        node * p = wToNode( cur_window, unq );
                        graphnodes.push_back( p );
                    }
                    break;

            }
            extend( cur_window );
            delete cur_window;
            cur_window = NULL;

        }
    }
    //Delete graph nodes
    void delNodes(){
        std::vector<node *>::iterator itr = graphnodes.begin();

        while( itr!= graphnodes.end() ){
            delete (*itr);
            itr++;
        }

        graphnodes.clear();
        seednodes.clear();
    }
    //Whether the given window is an edge
    std::vector<unsigned int> getUnique( window * w ){


        std::list< unsigned int > sts;
        std::vector< unsigned int > unique;
        sts.clear();

        sts.push_back( grid[w->row][w->col] );
        sts.push_back( grid[w->row][w->col + 1] );
        sts.push_back( grid[w->row + 1 ][w->col + 1] );
        sts.push_back( grid[w->row + 1 ][w->col] );

        while( !sts.empty( ) ){
            unsigned int front = sts.front( );
            unique.push_back(front);
            sts.remove(front);
        }
        return unique;

    }
    //Window to Node
    node * wToNode( window * w, std::vector<unsigned int> & unique ){

        node * n = new node();
        n->x = (double)(w->col) + 0.5;
        n->y = size_y - (double)(w->row) - 0.5;
        n->adj = unique;

        return n;
    }
    void extend( window * w ){

        window * east = new window;
        east->row = w->row;
        east->col = w->col+1;

        window * west = new window;
        west->row = w->row;
        west->col = w->col-1;

        window * north = new window;
        north->row = w->row-1;
        north->col = w->col;

        window * south = new window;
        south->row = w->row+1;
        south->col = w->col;

        wstack.push( north );
        wstack.push( east );
        wstack.push( south );
        wstack.push( west );

    }
    std::vector<node *> getNodes(){
        return graphnodes;
    }

    std::vector<node *> getSeeds(){
        return seednodes;
    }
    void printNodes(){

        std::vector<node *>::iterator itr = seednodes.begin();
        std::cout << "Seed Nodes" << std::endl;
        while( itr != seednodes.end() ){
            std::cout << "SNode[" << (*itr)->x << "," << (*itr)->y << "]" << std::endl;
            itr++;
        }

        itr = graphnodes.begin();
        std::cout << "Graph Nodes" << std::endl;
        while( itr != graphnodes.end() ){
            std::cout << "Node[" << (*itr)->x << "," << (*itr)->y << "]" << std::endl;
            itr++;
        }
    }
    vg_graph_data * getGraph( ){

            assert( graphnodes.size() > 0 && seednodes.size() > 0 );

            vg_graph_data * data = new vg_graph_data();

            data->locations = new location[graphnodes.size()];

            // What ToDO with edges...
            //data->edges = new std::pair< unsigned int, unsigned int>[graphnodes.size()];

            std::vector<double> tmp_edge_weights;
            std::vector< std::pair<unsigned int, unsigned int> > tmp_edges;


            data->adj_list = new vg_graph(graphnodes.size());


            vg_weightmap weightmap = get(edge_weight, *data->adj_list);

            double min_weight = VG_DEFAULT_EDGEWEIGHT;
            double max_weight = 0;

            for( int i = 0; i < graphnodes.size(); i++ ){
                    for( int j = i + 1; j < graphnodes.size(); j++ ){
                        if(connected(graphnodes[i], graphnodes[j])){
                            //std::cout << "Edge Found: " << i << ", " << j <<  std::endl;
                            vg_edge_desc e; bool inserted;
                            tie(e, inserted) = add_edge(i,j, *data->adj_list);

                            double xx = graphnodes[j]->x - graphnodes[i]->x;
                            double yy = graphnodes[j]->y - graphnodes[i]->y;
                            double d = xx*xx+yy*yy;

                            double w = sqrt(d);
                            if( w > max_weight ) max_weight = w;
                            if( w < min_weight ) min_weight = w;
                            weightmap[e] = w;
                            tmp_edge_weights.push_back(w);
                            tmp_edges.push_back( std::pair<unsigned int,unsigned int>(i,j) );
                            //std::cout << "Distance " << i << ", " << j << " is " << weightmap[e] <<   std::endl;
                        }
                    }
                // Do the stuff for all vertices.
                location loc;
                loc.x = graphnodes[i]->x;
                loc.y = graphnodes[i]->y;
                data->locations[i]=loc;
            }
            data->weights = new double[ tmp_edge_weights.size() ];
            for( int i = 0; i < tmp_edge_weights.size(); i++ ){
                data->weights[i] = tmp_edge_weights.at(i);
            }
            data->edges = new std::pair< unsigned int, unsigned int >[ tmp_edges.size() ];
            for( int i = 0; i < tmp_edges.size(); i++ ){
                data->edges[i] = tmp_edges.at(i);
            }

            data->numEdges = tmp_edges.size();
            data->numNodes = graphnodes.size();
            data->numSeed = seednodes.size();
            data->maxWeight = max_weight;
            data->minWeight = min_weight;

            return data;
    }

    //Whether the given two nodes are connected with an edge
    bool connected( node * n1, node * n2 ){

        int shared = 0;
        for( int i = 0; i < n1->adj.size(); i++ ){
            for( int j = 0; j < n2->adj.size(); j++ ){
                if( n1->adj.at(i) == n2->adj.at(j) ){
                    shared++;
                    break;
                }
            }
        }

        return shared >= 2;
    }

    //Get neighbors eklenecek


};
