//Autor: Ate Akaydn
//Date: 12.05.2008
/*
	This is the line site class which represents a 2d line voronoi site
	Only rasterization method is altered to raster a tent in between the source and destination points of the line
	Rasterization method must take a display list of type tent as the first element in order to work correctly
	tent type display list must be a unit length line on x axis originated from origin otherwise it will not be
	properly transformed by the rasterization routine

	Lines possess pointers to the source and destination points hence points are shared and they form the basis. Also line rasterization
	routine only rasterizes the tent so cones should also be rasterized by calling necessary routines for the points
*/

#ifndef VG_LINE2D_H
#define VG_LINE2D_H

#include "vg_site2d.h"
#include "vg_point2d.h"
#include "vg_constants.h"


class VG_Line2d : public VG_Site2d {

protected:

	VG_Point2d * pts[2];

public:

	//Default Constructor 1
	VG_Line2d( unsigned int st_id ):VG_Site2d(st_id){
        info = std::string( "line2d" );
		pts[0] = NULL;
		pts[1] = NULL;
	}
	//Default Constructor 2
	VG_Line2d( VG_Point2d * src, VG_Point2d * dst, unsigned int st_id ):VG_Site2d(st_id){
        info = std::string( "line2d" );
		pts[0] = src;
		pts[1] = dst;
		pts[0]->setSiteId(st_id);
		pts[1]->setSiteId(st_id);
	}

	//Copy Constructor
	VG_Line2d( const VG_Line2d & ln):VG_Site2d(ln.getSiteId() ){
		pts[0] = ln.getSrc();
		pts[1] = ln.getDst();
		pts[0]->setSiteId(site_id);
		pts[1]->setSiteId(site_id);
	}

	//Deconstructor
	~VG_Line2d(){
	}

	//Rasterization method of the site
	//Takes as single parameter of display list handles where the handles must have the following format
	//cones[0]: display list of a proper cone to be rasterized for vertices
	//cones[1]: display list of a proper tent to be rasterized for edges
	//cones[2..]: display lists for special cases... such as curves
	bool raster( const GLuint * cones, const VG_Decoder * decoder ){
		rasterEnds( cones, decoder );
		rasterBase( cones, decoder );
		return true;
	}

	//Rasters the end points of the line
	bool rasterEnds( const GLuint * cones, const VG_Decoder * decoder ){
		pts[0]->raster( cones,decoder );
		pts[1]->raster( cones,decoder );
		return true;
	}

	//Rasters the base of the line
	bool rasterBase( const GLuint * cones, const VG_Decoder * decoder ){

		assert( pts[0] != NULL && pts[1] != NULL );

		glLineWidth( VG_DEFAULT_LINEWIDTH );
		VG_Color clr = decoder->decodeColor( site_id );
		glColor3ub( clr.r, clr.g, clr.b );

		//First derive the angle(in degrees) in between source and distance
		double angle = atan2( pts[1]->getY() - pts[0]->getY(), pts[1]->getX() - pts[0]->getX() ) * 180.0/M_PI;

		//Then derive the distance in between the points
		GLfloat dst = static_cast<GLfloat>( pow( pow( (double)( pts[1]->getX() - pts[0]->getX() ), 2.0 ) + pow( (double)( pts[1]->getY() - pts[0]->getY() ), 2 ) , 0.5 ) );

		//Assuming first display list is of type cones
		glPushMatrix();
		glLoadIdentity();

		glTranslatef( pts[0]->getX(), pts[0]->getY(), 0.0 );

		glRotatef( angle, 0.0, 0.0, 1.0 ); //Burda z -1.0 olabilir

		//Scale burda sadece x ekseni boyunca olabilir
		glScalef( dst, 1.0, 1.0 );

		glCallList( cones[1] );

		glPopMatrix();

		return true;
	}

	bool draw( double r, double err, const VG_Decoder * decoder ){
		//TODO: will be implemented
		return true;
	}
	//Get Methods for coordinates
	VG_Point2d * getSrc() const{
		return pts[0];
	}
	VG_Point2d * getDst() const{
		return pts[1];
	}
	VG_Point2d * getPoint( unsigned short ind ) const{
		assert( ind >= 0 && ind < 2 );
		return pts[ind];
	}

	//Set Methods for coordinates
	void setSrc( VG_Point2d * p ){
		pts[0] = p;
	}
	void setDst( VG_Point2d * p ){
		pts[1] = p;
	}
	void setPoint( unsigned short ind, VG_Point2d * p ){
		assert( ind >= 0 && ind < 2 );
		pts[ind] = p;
	}
	void setSiteId( unsigned int st_id ){
		this->site_id = st_id;
		pts[0]->setSiteId(st_id);
		pts[1]->setSiteId(st_id);
	}

	//Set Methods for coordinates
	VG_Point2d * getSrc(){
		return pts[0];
	}

	VG_Point2d * getDst(){
		return pts[1];
	}
	VG_Point2d * getPoint( unsigned short ind ){
		assert( ind >= 0 && ind < 2 );
		return pts[ind];
	}
    //Outputs the site information to the given file stream
	void toFileStream(  std::ofstream & fstrm ){
            fstrm << ( info.data() ) << std::endl;
            fstrm << pts[0]->getDim( 0 ) << "\t" << pts[0]->getDim( 1 ) << std::endl;
            fstrm << pts[1]->getDim( 0 ) << "\t" << pts[1]->getDim( 1 ) << std::endl;
    };



};

#endif
