//Autor: Ate Akaydn
//Date: 11.05.2008
/*
	This is the point site class which represents a 2d point voronoi site
	Only rasterization method is altered to raster a cone located at this sites position
	Rasterization method must take a display list of type cone as the first element in order to work correctly
*/

#ifndef VG_POINT2D_H
#define VG_POINT2D_H

#include "vg_site2d.h"

class VG_Point2d : public VG_Site2d {

protected:

	GLfloat dim[2];

public:

	//Default Constructor
	VG_Point2d( GLfloat x, GLfloat y, unsigned int st_id ):VG_Site2d(st_id){
        info = std::string( "point2d" );
		dim[0] = x;
		dim[1] = y;
	}

	//Copy Constructor
	VG_Point2d( const VG_Point2d & pt):VG_Site2d(pt.getSiteId() ){
		dim[0] = pt.getX();
		dim[1] = pt.getY();
	}

	//Deconstructor
	~VG_Point2d(){

	}

	//Rasterization method of the site
	//Takes as single parameter of display list handles where the handles must have the following format
	//cones[0]: display list of a proper cone to be rasterized for vertices
	//cones[1]: display list of a proper tent to be rasterized for edges
	//cones[2..]: display lists for special cases... such as curves
	bool raster( const GLuint * cones, const VG_Decoder * decoder ){

		glPointSize( VG_DEFAULT_POINTSIZE );
		VG_Color clr = decoder->decodeColor( site_id );
		glColor3ub( clr.r, clr.g, clr.b );
		//Assuming first display list is of type cones
		glPushMatrix();
		glLoadIdentity();
		glTranslatef( dim[0], dim[1], 0 );
		glCallList( cones[0] );
		glPopMatrix();

		return true;

	}

	bool draw( double r, double er_thres, const VG_Decoder * decoder ){

		assert( er_thres > 0 );

		VG_Color clr = decoder->decodeColor( site_id );
		glColor3ub( clr.r, clr.g, clr.b );

		glPushMatrix();
		glLoadIdentity();

		//Step is the unit angle in between each triangle
		GLfloat stp =  2 * acos( (r-er_thres)/r );

		glTranslatef( dim[0], dim[1], 0.0 );



		//Draw the cone mesh to the list
		glBegin( GL_TRIANGLE_FAN );

			glVertex3f( 0.0 , 0.0 ,0.0 );

			for( GLfloat angle = 0; angle <= 360.0; angle+=stp ){

				glVertex3f( r*cos(angle*M_PI/180), r*sin(angle*M_PI/180), -r );

			}

		glEnd();

		glBegin( GL_POINTS );

			glPointSize( VG_DEFAULT_POINTSIZE );
			glColor3ub( 0, 0, 0 );
			glVertex3f( 0.0, 0.0, 0.0 );

		glEnd();

		glPopMatrix();

		return true;

	}
	//Get Methods for coordinates
	GLfloat getX() const{
		return dim[0];
	}
	GLfloat getY() const{
		return dim[1];
	}
	GLfloat getDim( unsigned short ind ) const{
		assert( ind >= 0 && ind < 2 );
		return dim[ind];
	}

	//Set Methods for coordinates
	void setX(GLfloat x ){
		dim[0] = x;
	}
	void setY(GLfloat y ){
		dim[1] = y;
	}
	void setDim( unsigned short ind, GLfloat value ){
		assert( ind >= 0 && ind < 2 );
		dim[ind] = value;
	}

    //Outputs the site information to the given file stream
	void toFileStream(  std::ofstream & fstrm ){
            fstrm << ( info.data() ) << std::endl;
            fstrm << dim[0] << "\t" << dim[1] << std::endl;
    };


};

#endif
