//Autor: Ate Akaydn
//Date: 12.05.2008
/*
	This is the polygon site class which represents a 2d polygon voronoi site
	Polygons possess pointers to the corners points and edges
*/

#ifndef VG_POLYGON2D_H
#define VG_POLYGON2D_H

#include "vg_site2d.h"
#include "vg_line2d.h"
#include <vector>

class VG_Polygon2d : public VG_Site2d {
	
protected:

	std::vector<VG_Point2d*> pts;

public:

	//Default Constructor 1
	VG_Polygon2d( unsigned int st_id ):VG_Site2d(st_id){
        info = std::string( "polygon2d" );
		pts.clear();
	}
	//Default Constructor 2
	VG_Polygon2d( std::vector<VG_Point2d *> & tpts, unsigned int st_id ):VG_Site2d(st_id){
        info = std::string( "polygon2d" );                  
		pts.clear();
		std::vector<VG_Point2d *>::iterator it = tpts.begin();
		while( it!= tpts.end() ){
			pts.push_back( *it );
			(*it)->setSiteId( site_id );
			it++;
		}
	}

	//Copy Constructor
	VG_Polygon2d( const VG_Polygon2d & ln):VG_Site2d(ln.getSiteId() ){
        info = std::string( "polygon2d" );                  
		pts.clear();
		std::vector<VG_Point2d *> tmp = ln.getPoints();
		std::vector<VG_Point2d *>::iterator it = tmp.begin();
		while( it != tmp.end() ){
			pts.push_back( *it );
			(*it)->setSiteId( site_id );
			it++;
		}
	}

	//Deconstructor
	~VG_Polygon2d(){
	}

	//Rasterization method of the site
	//Takes as single parameter of display list handles where the handles must have the following format
	//cones[0]: display list of a proper cone to be rasterized for vertices
	//cones[1]: display list of a proper tent to be rasterized for edges
	//cones[2..]: display lists for special cases... such as curves
	bool raster( const GLuint * cones, const VG_Decoder * decoder ){
		rasterVertices( cones, decoder );
		rasterEdges( cones, decoder );
		return true;
	}

	//Rasters the vertices of the line
	bool rasterVertices( const GLuint * cones, const VG_Decoder * decoder ){
		
		if( pts.size() <= 0 ) return false;

		std::vector<VG_Point2d*>::iterator it = pts.begin();
		while( it != pts.end() ){
			(*it)->raster( cones, decoder );
			it++;
		}
		return true;
	}

	//Rasters the edges of the polyong
	bool rasterEdges( const GLuint * cones, const VG_Decoder * decoder ){
		
		if( pts.size() <= 1 ) return false;
		else if( pts.size() == 2 ){
			VG_Point2d * p1 = pts.front();
			VG_Point2d * p2 = pts.back();
			VG_Line2d line( p1,p2, site_id );
			line.rasterBase( cones, decoder );
		}
		else{
			std::vector<VG_Point2d*>::iterator it = pts.begin( );
			while( it != pts.end() ){
				VG_Point2d * p1 = (*it);
				VG_Point2d * p2;
				if( (it+1) != pts.end() )
					p2 = *(it+1);
				else
					p2 = pts.front();

				VG_Line2d line( p1,p2, site_id );
				line.rasterBase( cones, decoder );
				it++;
			}
		}
		return true;
	}

	bool draw( double r, double err, const VG_Decoder * decoder ){
		//TODO: will be implemented
		return true;
	}
	VG_Point2d * getPoint( unsigned short ind ) const{
		assert( ind >= 0 && ind < pts.size() );
		return pts.at(ind);
	}

	//Add method for point
	void addPoint( VG_Point2d * p ){
		p->setSiteId( site_id );
		pts.push_back(p);
	}
	//Remove method for point
	void removePoint( unsigned int ind ){
		assert( ind >= 0 && ind < pts.size() );
		
	}
	//Returns points
	std::vector< VG_Point2d * > getPoints() const{
		return pts;
	}
    //Outputs the site information to the given file stream
	void toFileStream(  std::ofstream & fstrm ){
            fstrm << ( info.data() ) << std::endl;  
            fstrm << pts.size() << std::endl;        
    		std::vector<VG_Point2d*>::iterator it = pts.begin();
    		while( it != pts.end() ){
    			fstrm << (*it)->getDim(0) << "\t" << (*it)->getDim(1) << std::endl;
    			it++;
    		}           
    };	


};

#endif
