//Autor: Ate Akaydn
//Date: 11.05.2008
/*
	This is the basis, abstract voronoi site class for vorgen project
	It has a site id and a raster method which must be overwritten by the descendants.
	Raster method takes the display lists representing cones and tents and each class extending this one
	is reponsible for its proper rasterization during the generation of the 2d voronoi diagram.
*/

#ifndef VG_SITE2D_H
#define VG_SITE2D_H

#include <windows.h>
#include <gl.h>
#include <glu.h>
#include <string>
//#include <gl/glaux.h>
#include <assert.h>
#include "vg_decoder.h"
#include <iostream>
#include <fstream>
//#include "vg_constants.h"

class VG_Site2d{
	
protected:

    std::string info;
	unsigned int site_id;

public:

	//Default Constructor
	VG_Site2d( unsigned int st_id ):site_id(st_id){
         info = std::string("site2d");
	}

	//Copy Constructor
	VG_Site2d( const VG_Site2d & st){
		site_id = st.getSiteId();
	}

	//Deconstructor
	virtual ~VG_Site2d(){

	}

	//Rasterization method of the site
	//Takes as single parameter of display list handles where the handles must have the following format
	//cones[0]: display list of a proper cone to be rasterized for vertices
	//cones[1]: display list of a proper tent to be rasterized for edges
	//cones[2..]: display lists for special cases... such as curves
	virtual bool raster( const GLuint * cones, const VG_Decoder * decoder )= 0;

	//Draws the distance function of the site without using display lists
	virtual bool draw( double r, double err, const VG_Decoder * decoder )= 0;


    //Outputs the site information to the given file stream
	virtual void toFileStream(  std::ofstream & fstrm )=0;
	
	void setSiteId( unsigned int site_id ){
		this->site_id = site_id;
	}

	unsigned int getSiteId() const{
		return site_id;
	}
	
	//Set information for this site
	void setInfo( char * str ){
         info = std::string( str ); 
    }

};

#endif
