//Autor: Ate Akaydn, Gnen Ercan
//Date: 27.05.2008
/*
    Visualizer GUI to visualize and to construct related buffer and site data
*/

#include <windows.h>
#include <cmath>
#include <vector>
#include <gl.h>
#include <glu.h>
#include <glut.h>
//#include <gl/glaux.h>
#include <glui.h>

#include "vg_constants.h"
#include "vg_vorgen2d.h"
#include "vg_point2d.h"
#include "vg_line2d.h"
#include "vg_polygon2d.h"
#include "vg_graph.h"
#include <cstdlib>
#include <time.h>

#include <iostream>
#include <fstream>

//Drawing modes
enum drawingMode{ d_points, d_lines, d_polygons, d_idle } mode;
bool change_flag = false;
bool draw_graph = false;
bool draw_search_path = false;

//Temporary sites
VG_Line2d * tmp_line = NULL;
VG_Polygon2d * tmp_poly = NULL;

//Voronoi generator instance
VG_Vorgen2d * vorgen;

//Current c_buffer data
GLubyte * c_buffer;
//Current z_buffer data
GLfloat * z_buffer;

//Static Graph
vg_graph_data * static_graph;

//AStar search results
std::vector<vg_vertex> s_path;
double s_distance;

int main_window;
//GLUI Objects
GLUI * master_window;

GLUI_Rollout * primitivesRollout;
GLUI_Rollout * toolsRollout;
GLUI_Rollout * ioopRollout;
GLUI_Rollout * graphRollout;
GLUI_Rollout * automatedRollout;
GLUI_EditText * automatedText;

//Astar search init and goal node ids
GLUI_EditText * initNodeText;
GLUI_EditText * goalNodeText;

//Text fields for file Outputting
GLUI_EditText * renderFileText;
GLUI_EditText * graphFileText;

//Text fields for specifying rectangle corners
GLUI_EditText * topCornerXText;
GLUI_EditText * topCornerYText;
GLUI_EditText * bottomCornerXText;
GLUI_EditText * bottomCornerYText;

//Callback functions
void displayFunc( void );
void mouseClickFunc( int, int, int, int);
void idleFunc( void );
void initFunc( void );
void mousePassiveMotionFunc( int,int );
void reshapeFunc( GLsizei w, GLsizei h);
void handle_button_clicks( int );

double aStarSearch(unsigned int init, unsigned int goal, std::vector<vg_vertex> & );

int main( int argc, char ** argv ){

	//glutInit( &argc, argv );
	change_flag = false;

	int size_x = VG_DEFAULT_WINDOW_SIZE, size_y = VG_DEFAULT_WINDOW_SIZE;
	double err_thres = VG_DEFAULT_ERROR_THRESHOLD;
	unsigned int dec_size = VG_DEFAULT_DECODER_SIZE;

    switch( argc ){
            case(2):{
                    sscanf( argv[1], "%lf", &err_thres );
            }
                    break;
            case(3):{
                    sscanf( argv[1], "%lf", &err_thres );
                    sscanf( argv[2], "%d", &size_x );
                    size_y = size_x;
            }
                    break;
            case(4):{
                    sscanf( argv[1], "%lf", &err_thres );
                    sscanf( argv[2], "%d", &size_x );
                    sscanf( argv[3], "%d", &size_y );
            }
                    break;
            case(5):{
                    sscanf( argv[1], "%lf", &err_thres );
                    sscanf( argv[2], "%d", &size_x );
                    sscanf( argv[3], "%d", &size_y );
                    sscanf( argv[4], "%d", &dec_size );
            }
                    break;
            default:
                    break;

    }



	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	main_window = glutCreateWindow( "Vorgen Visualizer" );
	glutPositionWindow( 100,100 );
	glutReshapeWindow( size_x,size_y  );

	//Master Menu window initialization
	master_window = GLUI_Master.create_glui( "Menu" , 0 , 720, 100);
	master_window->set_main_gfx_window( main_window );


	//Primitives rollout initialization
	primitivesRollout = master_window->add_rollout( "Primitives" , true );

	master_window->add_button_to_panel( primitivesRollout, "Points", 1, handle_button_clicks  );

	master_window->add_button_to_panel( primitivesRollout, "Lines", 2, handle_button_clicks  );
	master_window->add_button_to_panel( primitivesRollout, "Polygons", 3, handle_button_clicks  );


	//Automated rollout initialization
	automatedRollout = master_window->add_rollout( "Automated" , true );
	automatedText = master_window->add_edittext_to_panel( automatedRollout, "Number of Points" );
	master_window->add_button_to_panel( automatedRollout, "Generate", 4, handle_button_clicks  );

	topCornerXText = master_window->add_edittext_to_panel( automatedRollout, "Top Left X:" );
	topCornerYText = master_window->add_edittext_to_panel( automatedRollout, "Top Left Y:" );

	bottomCornerXText = master_window->add_edittext_to_panel( automatedRollout, "Bottom Right X:" );
	bottomCornerYText = master_window->add_edittext_to_panel( automatedRollout, "Bottom Right Y:" );

    master_window->add_button_to_panel( automatedRollout, "Rectangle", 5, handle_button_clicks  );
    master_window->add_button_to_panel( automatedRollout, "Lines", 6, handle_button_clicks  );

	//Tools rollout initialization
	toolsRollout = master_window->add_rollout( "Tools" , true );
	master_window->add_button_to_panel( toolsRollout, "Render", 7, handle_button_clicks  );
	master_window->add_button_to_panel( toolsRollout, "Clear", 8, handle_button_clicks  );
	master_window->add_button_to_panel( toolsRollout, "Read Framebuffer", 9, handle_button_clicks  );

	//Tools rollout initialization
	graphRollout = master_window->add_rollout( "Graphs Tools" , true );
	master_window->add_button_to_panel( graphRollout, "Construct Graph", 10, handle_button_clicks  );
	initNodeText = master_window->add_edittext_to_panel( graphRollout, "Init Node:" );
    goalNodeText = master_window->add_edittext_to_panel( graphRollout, "Goal Node:" );
    master_window->add_button_to_panel( graphRollout, "A Star Search", 11, handle_button_clicks  );

    ioopRollout = master_window->add_rollout( "IO Operations" , true );
	renderFileText = master_window->add_edittext_to_panel( ioopRollout, "File Name" );
	master_window->add_button_to_panel( ioopRollout, "Write C-Buffer", 12, handle_button_clicks  );
    master_window->add_button_to_panel( ioopRollout, "Write Z-Buffer", 13, handle_button_clicks  );
    master_window->add_button_to_panel( ioopRollout, "Read Vorgen", 14, handle_button_clicks  );
	master_window->add_button_to_panel( ioopRollout, "Write Vorgen", 15, handle_button_clicks  );
	master_window->add_button_to_panel( ioopRollout, "Write Graph", 16, handle_button_clicks  );


	GLUI_Master.set_glutDisplayFunc( displayFunc );
    GLUI_Master.set_glutReshapeFunc( reshapeFunc );
	GLUI_Master.set_glutIdleFunc( idleFunc );
	glutPassiveMotionFunc( mousePassiveMotionFunc );
	GLUI_Master.set_glutMouseFunc( mouseClickFunc );


	vorgen = new VG_Vorgen2d( size_x, size_y, err_thres, dec_size );

	initFunc();
	glutMainLoop();

}

//Initializing clear color and orthonormal projection
void initFunc(void)
{

	glClearColor( 1.0, 1.0, 1.0, 1.0 );
	glShadeModel( GL_FLAT );
	glEnable( GL_DEPTH_TEST );

	//This might be problematic.
	glPixelStorei( GL_PACK_ALIGNMENT, 1 );

	unsigned int size_x = vorgen->getSizeX();
	unsigned int size_y = vorgen->getSizeY();

	//Length of the diagonal
	double depth = pow( pow( size_x, 2.0 ) + pow( size_y, 2.0 ), 0.5 );

	glClearColor( 1.0, 1.0, 1.0, 1.0 );

	//Enable depth test
	glEnable( GL_DEPTH_TEST );

	//Load Projection matrix
	glMatrixMode( GL_PROJECTION );

	//Orthonormal Projection
	glOrtho( 0, size_x, 0, size_y, 0, depth );
	glViewport( 0, 0,size_x, size_y );

	//Load model matrix
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

}
void displayFunc( void ){

	if( change_flag ){
	    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        clock_t start = clock();

    	if( draw_search_path ){


    	    glLineWidth( VG_DEFAULT_PATHWIDTH );
    	    glBegin( GL_LINES );

            glColor3f( 0.2, 1.0, 0.2 );
            for( int i = 0; i < s_path.size() - 1; i++ ){

                glVertex3f( static_graph->locations[s_path.at(i)].x, static_graph->locations[s_path.at(i)].y, 0.0 );

                glVertex3f( static_graph->locations[s_path.at(i+1)].x, static_graph->locations[s_path.at(i+1)].y, 0.0 );

    	    }

    	    glEnd( );
            draw_search_path = false;
    	}

    	if( draw_graph & static_graph != NULL ){
            glPointSize( VG_DEFAULT_NODESIZE );
            glBegin( GL_POINTS );

            for( int i = 0; i < static_graph->numNodes; i++ ){

                if( i < static_graph->numSeed ){
                    glColor3f( 0.6, 0.6, 1.0 );
                }
                else{
                    glColor3f( 1.0, 0.6, 0.6 );
                }

                glVertex3f( static_graph->locations[i].x, static_graph->locations[i].y, 0.0 );

            }
            glEnd( );

            //Draw edges
            glLineWidth( VG_DEFAULT_LINEWIDTH );
            glBegin( GL_LINES );


            for( int j = 0; j < static_graph->numEdges; j++ ){

                double c = static_graph->weights[j]/(static_graph->maxWeight - static_graph->minWeight );
                glColor3f( c, c, c );
                unsigned int first = static_graph->edges[j].first;
                unsigned int second = static_graph->edges[j].second;
                glVertex3f( static_graph->locations[first].x, static_graph->locations[first].y, 0.0 );
                glVertex3f( static_graph->locations[second].x, static_graph->locations[second].y, 0.0 );

            }

            glEnd();

            draw_graph = false;
    	}

    	//Draw Vorgen
    	vorgen->draw_d( );

    	glutSwapBuffers();
    	glFinish();

    	clock_t finish = clock();

    	std::cout << "Generation Time: " << ( (double)(finish-start)/CLOCKS_PER_SEC ) << std::endl;
    	change_flag = false;
     }

}
void idleFunc( void ){
    if( glutGetWindow() != main_window )
        glutSetWindow(main_window);
    glutPostRedisplay();
}

void reshapeFunc( GLsizei w, GLsizei h){

	unsigned int size_x = vorgen->getSizeX();
	unsigned int size_y = vorgen->getSizeY();

	//Length of the diagonal
	double depth = pow( pow( size_x, 2.0 ) + pow( size_y, 2.0 ), 0.5 );

	//Load Projection matrix
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );

	//Orthonormal Projection
	glOrtho( 0, w, 0, h, 0, depth );
	glViewport( 0, 0,w, h );

	//Load model matrix
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

	change_flag = true;
    glutPostRedisplay();
}


void handle_button_clicks( int id ){

     switch( id ){
             case(1): //Create Points
                      mode = d_points;
             break;
             case(2): //Create Lines
                      mode = d_lines;
             break;
             case(3): //Create Polygons
                      mode = d_polygons;
             break;
             case(4):
             { //Automated Generation
                      int numRandom = atoi( automatedText->get_text() );
                      std::cout << numRandom << std::endl;
                      srand( time(NULL) );
                      VG_Point2d * tmp;
                      for( int i = 0; i < numRandom; i++ ){
                           tmp = new VG_Point2d( rand()%vorgen->getSizeX(), rand()%vorgen->getSizeY(), 0 );
                           vorgen->addSite( tmp );
                      }
                      change_flag = true;
             }
             break;
             case(5): //Add rectangle
             {
                int topLeftX = atoi( topCornerXText->get_text() );
                int topLeftY = atoi(topCornerYText->get_text() );
                int bottomRightX = atoi(bottomCornerXText->get_text() );
                int bottomRightY = atoi(bottomCornerYText->get_text() );

                //Construct the triangle
                VG_Polygon2d * rect = new VG_Polygon2d( vorgen->getNumSites( ) );

                rect->addPoint( new VG_Point2d( GLfloat(topLeftX), GLfloat(bottomRightY), 0 ) );
                rect->addPoint( new VG_Point2d( GLfloat(bottomRightX), GLfloat(bottomRightY), 1 ) );
                rect->addPoint( new VG_Point2d( GLfloat(bottomRightX), GLfloat(topLeftY), 2 ) );
                rect->addPoint( new VG_Point2d( GLfloat(topLeftX), GLfloat(topLeftY), 3 ) );

                vorgen->addSite( rect );

                change_flag = true;
             }
             break;
             case(6): //Add Lines
             {
                int topLeftX = atoi( topCornerXText->get_text() );
                int topLeftY = atoi(topCornerYText->get_text() );
                int bottomRightX = atoi(bottomCornerXText->get_text() );
                int bottomRightY = atoi(bottomCornerYText->get_text() );

                //Construct the triangle
                VG_Line2d * line = new VG_Line2d( new VG_Point2d( GLfloat(topLeftX), GLfloat(bottomRightY), 0 ), new VG_Point2d( GLfloat(bottomRightX), GLfloat(bottomRightY), 1 ), vorgen->getNumSites( ) );
                vorgen->addSite(line);
                line = new VG_Line2d( new VG_Point2d( GLfloat(bottomRightX), GLfloat(bottomRightY), 2 ), new VG_Point2d( GLfloat(bottomRightX), GLfloat(topLeftY), 3 ), vorgen->getNumSites( ) );
                vorgen->addSite(line);
                line = new VG_Line2d( new VG_Point2d( GLfloat(bottomRightX), GLfloat(topLeftY), 4 ), new VG_Point2d( GLfloat(topLeftX), GLfloat(topLeftY), 5 ), vorgen->getNumSites( ) );
                vorgen->addSite( line );
                line = new VG_Line2d( new VG_Point2d( GLfloat(topLeftX), GLfloat(topLeftY), 6 ), new VG_Point2d( GLfloat(topLeftX), GLfloat(bottomRightY), 7 ) , vorgen->getNumSites( ) );
                vorgen->addSite( line );
                change_flag = true;
             }
             break;
             case(7): //Render
                      change_flag = true;
             break;
             case(8): //Clear
             {
                if( c_buffer != NULL ){
                    std::cout << "Deleting existing color-buffer..." << std::endl;
                    delete[] c_buffer;
                    std::cout << "Done." << std::endl;
                    c_buffer = NULL;
                }
                if( z_buffer != NULL ){
                    std::cout << "Deleting existing z-buffer..." << std::endl;
                    delete[] z_buffer;
                    std::cout << "Done." << std::endl;
                    z_buffer = NULL;
                }
                if( static_graph != NULL ){
                        //Delete existing static graph
                        delete[] static_graph->locations;
                        delete[] static_graph->edges;
                        delete static_graph->adj_list;
                        delete[] static_graph->weights;
                        delete static_graph;
                        static_graph = NULL;
                }
                vorgen->deleteSites();
                vorgen->clearSites();
                change_flag = true;

             }
             break;
             case(9): //Read Buffer
             {
                if( c_buffer != NULL ){
                    std::cout << "Deleting existing color-buffer..." << std::endl;
                    delete[] c_buffer;
                    std::cout << "Done." << std::endl;
                    c_buffer = NULL;
                }
                if( z_buffer != NULL ){
                    std::cout << "Deleting existing z-buffer..." << std::endl;
                    delete[] z_buffer;
                    std::cout << "Done." << std::endl;
                    z_buffer = NULL;
                }

        		unsigned int size_x = vorgen->getSizeX( );
        		unsigned int size_y = vorgen->getSizeY( );

        		c_buffer = new GLubyte[ static_cast<int>(3*size_x*size_y) ];
        		z_buffer = new GLfloat[ static_cast<int>(size_x*size_y) ];

        		glReadBuffer(GL_FRONT);

        		std::cout << "Reading color-buffer..." << std::endl;
        		glReadPixels( 0,0,size_x,size_y,GL_RGB,GL_UNSIGNED_BYTE,c_buffer );
        		std::cout << "Done." << std::endl;

        		std::cout << "Reading z-buffer..." << std::endl;
        		glReadPixels( 0,0,size_x,size_y,GL_DEPTH_COMPONENT, GL_FLOAT, z_buffer );
        		std::cout << "Done." << std::endl;

             }
             break;
             case(10): //Construct Graph
             {

        		 unsigned int size_x = vorgen->getSizeX( );
                 unsigned int size_y = vorgen->getSizeY( );

                 if( c_buffer != NULL ){

                       std::cout << "Constructing Graph..." << std::endl;

                       VG_Decoder* dc = vorgen->getDecoder();

                        //bool exists = false;
                        //VG_Color tmp_color = VG_Color( static_cast<GLfloat>(c_buffer[0]/255.0 ), static_cast<GLfloat>(c_buffer[1]/255.0 ), static_cast<GLfloat>(c_buffer[2]/255.0 ) );
                        //dc->encodeIndex( tmp_color, exists );
                        //std::cout << exists << std::endl;
                        unsigned int** grid = dc->encodeRGBBuffer( c_buffer, size_x, size_y );

                        VG_Graph graphBuilder( grid, size_x, size_y );

                        clock_t start = clock();



                        graphBuilder.findGraphNodes( );


                        clock_t finish = clock();
                        std::cout << "Graph Construction Time: " << ( (double)(finish-start)/CLOCKS_PER_SEC ) << std::endl;

                        draw_graph = true;
                        change_flag = true;

                        std::ofstream file( "site_buffer.txt", std::ofstream::out );

                        for( int y = 0; y < size_y; y++ ){
                            for( unsigned int x = 0; x < size_x; x++ ){
                                    file << grid[y][x];
                                    file << "\t";
                            }
                            file << std::endl;
                        }
                        file.close();

                        //If static graph was set previously delete it to avoid memory leak
                        if( static_graph != NULL ){
                                //Delete existing static graph
                                delete[] static_graph->locations;
                                delete[] static_graph->edges;
                                delete static_graph->adj_list;
                                delete[] static_graph->weights;
                                delete static_graph;
                                static_graph = NULL;
                        }
                        static_graph = graphBuilder.getGraph();

                        std::cout << "Done." << std::endl ;

                 }

             }
             break;
             case(11): //A* Star search
             {

                if( static_graph == NULL ){
                    std::cout << "ERROR: Static graph not initialized!" << std::endl;
                    break;
                }

                unsigned int init = atoi( initNodeText->get_text() );
                unsigned int goal = atoi( goalNodeText->get_text() );

                if( init < 0 || goal < 0 || init >= static_graph->numNodes || goal >= static_graph->numNodes ){
                        std::cout << "ERROR: Init&goal node indices invalid!" << std::endl;
                        break;
                }

                s_distance = aStarSearch(init, goal, s_path );

                draw_graph = true;
                draw_search_path = true;
                change_flag = true;

             }
             break;
             case(12): //Write Color - Buffer to file
             {
                if( c_buffer != NULL ){
                    std::cout << "Writing Color-Buffer to " << renderFileText->get_text() << "..." << std::endl ;

            		unsigned int size_x = vorgen->getSizeX( );
            		unsigned int size_y = vorgen->getSizeY( );
                    //Burda yazmak istedigimiz file olacak
                    //BMP dosyasina da yazilabilinir BMP writer bulursam
                    std::ofstream file( renderFileText->get_text(), std::ofstream::out );

            		for( int y = size_y - 1; y >= 0; y-- ){
            			for( unsigned int x = 0; x < size_x; x++ ){
            					file << "( R:";
            					file <<  static_cast<unsigned int>(c_buffer[static_cast<unsigned int>(y*size_x*3 + x*3 + 0)]);
            					file << ", G:";
            					file <<  static_cast<unsigned int>(c_buffer[static_cast<unsigned int>(y*size_x*3 + x*3 + 1)]);
            					file << ", B:";
            					file <<  static_cast<unsigned int>(c_buffer[static_cast<unsigned int>(y*size_x*3 + x*3 + 2)]);
            					file << " )";
            					file << "\t";

            			}
            			file << std::endl;
            		}
            		file.close();
                    std::cout << "Done." << std::endl ;
                }
                else{
                     std::cout << "ERROR: Color Buffer not loaded yet !" << std::endl;
                }
             }
             break;
             case(13): //Write Z-Buffer to file
             {
                if( z_buffer != NULL ){
                    std::cout << "Writing Z-Buffer to " << renderFileText->get_text() << "..." << std::endl ;

            		unsigned int size_x = vorgen->getSizeX( );
            		unsigned int size_y = vorgen->getSizeY( );
                    //Burda yazmak istedigimiz file olacak
                    //BMP dosyasina da yazilabilinir BMP writer bulursam
                    std::ofstream file( renderFileText->get_text(), std::ofstream::out );

                    file << size_x << "\t" << size_y << std::endl;

            		for( int y = size_y - 1; y >= 0; y-- ){
            			for( unsigned int x = 0; x < size_x; x++ ){
            					file <<  (z_buffer[static_cast<unsigned int>(y*size_x + x )]) << "\t";
            			}
            			file << std::endl;
            		}
            		file.close();
                    std::cout << "Done." << std::endl ;
                }
                else{
                     std::cout << "ERROR: Z-Buffer not loaded yet !" << std::endl;
                }
             }
             break;
             case(14): //Read Vorgen from file
             {
                std::cout << "Reading files from " << renderFileText->get_text() << "..." << std::endl ;

                std::ifstream file( renderFileText->get_text() );
                vorgen->fromFileStream( file );
                file.close();

                change_flag = true;
                std::cout << "Done." << std::endl ;

             }
             break;
             case(15): //Write Vorgen to file
             {
                    std::cout << "Writing Sites to " << renderFileText->get_text() << "..." << std::endl ;

                    std::ofstream file( renderFileText->get_text(), std::ofstream::out );
                    vorgen->toFileStream( file );
            		file
            		.close();
                    std::cout << "Done." << std::endl ;
             }
             break;
             case(16): //Write Graph to file
             {
                    if( static_graph == NULL ){
                        std::cout << "ERROR: Graph not initialized, cannot write!" << std::endl;
                    }
                    else{

                        std::cout << "Writing Graph to " << renderFileText->get_text() << "..." << std::endl ;
                        std::ofstream file( renderFileText->get_text(), std::ofstream::out );

                        //Write graph

                        //Graph Size
                        file << vorgen->getSizeX() << "\t" << vorgen->getSizeY() << std::endl;
                        //#Seeds
                        file << static_graph->numSeed << std::endl;

                        //#Nodes
                        file << static_graph->numNodes << std::endl;

                        //Node locations
                        for( int i = 0; i < static_graph->numNodes; i++ ){
                            file << static_graph->locations[i].x << "\t" << static_graph->locations[i].y << std::endl;
                        }

                        //#Edges
                        file << static_graph->numEdges << std::endl;

                        //Edge Node indices & weights
                        for( int e = 0 ; e < static_graph->numEdges; e++ ){
                            unsigned int first = static_graph->edges[e].first;
                            unsigned int second = static_graph->edges[e].second;

                            file << first << "\t" << second << "\t" << static_graph->weights[e] << std::endl;
                        }

                        //Minimum&Maximum Weight
                        file << static_graph->minWeight << "\t" <<  static_graph->maxWeight << std::endl;

                        file.close();
                        std::cout << "Done." << std::endl ;
                    }
             }
             break;

     }

}

void mouseClickFunc( int button, int state, int x, int y){

     switch( mode ){
             case( d_points ):
                   if( button == GLUT_LEFT_BUTTON && state == GLUT_UP ){
                       vorgen->addSite( new VG_Point2d( GLfloat(x), GLfloat(vorgen->getSizeY()-y), 0 ) );
                       change_flag = true;
                   }
                   else if( button = GLUT_RIGHT_BUTTON && state == GLUT_UP ){
                        mode = d_idle;
                   }
                   break;
             case( d_lines ):
                   if( button == GLUT_LEFT_BUTTON && state == GLUT_UP ){
                       if( tmp_line == NULL ){
                           tmp_line = new VG_Line2d( 0 );
                           tmp_line->setSrc( new VG_Point2d( GLfloat(x), GLfloat(vorgen->getSizeY()-y), 0 ) );
                       }
                       else{
                            tmp_line->setDst( new VG_Point2d( GLfloat(x), GLfloat(vorgen->getSizeY()-y), 0 ) );
                            tmp_line->setSiteId( vorgen->getNumSites( ) );
                            vorgen->addSite( tmp_line );
                            tmp_line = NULL;
                            change_flag = true;
                       }
                   }
                   else if( button = GLUT_RIGHT_BUTTON && state == GLUT_UP ){
                        if( tmp_line != NULL ){
                            delete tmp_line->getSrc( );
                            delete tmp_line;
                            tmp_line = NULL;
                        }
                        mode = d_idle;
                   }
                   break;
             case( d_polygons ):
                   if( button == GLUT_LEFT_BUTTON && state == GLUT_UP ){
                       if( tmp_poly == NULL ){
                           tmp_poly = new VG_Polygon2d( vorgen->getNumSites( ) );
                           tmp_poly->addPoint( new VG_Point2d( GLfloat(x), GLfloat(vorgen->getSizeY()-y), 0 ) );
                       }
                       else{
                           tmp_poly->addPoint( new VG_Point2d( GLfloat(x), GLfloat(vorgen->getSizeY()-y), 0 ) );
                       }
                   }
                   else if( button = GLUT_RIGHT_BUTTON && state == GLUT_UP ){
                        if( tmp_poly == NULL ){
                            mode = d_idle;
                        }
                        else{
                             vorgen->addSite( tmp_poly );
                             tmp_poly = NULL;
                             change_flag = true;
                        }

                   }
                   break;
             case( d_idle ):
                   break;
     }

}

double aStarSearch(unsigned int init, unsigned int goal, std::vector<vg_vertex> & shortest_path ) {

  shortest_path.clear();

  std::vector<vg_graph::vertex_descriptor> p(num_vertices(*static_graph->adj_list));
  std::vector<double> d(num_vertices(*static_graph->adj_list));
  try {
    // call astar named parameter interface
    astar_search
      (*static_graph->adj_list, vertex(init, *static_graph->adj_list),
       distance_heuristic<vg_graph, double, location*>
        (static_graph->locations, vertex(goal, *static_graph->adj_list)),
       predecessor_map(&p[0]).distance_map(&d[0]).
       visitor(astar_goal_visitor<vg_vertex>(vertex(goal, *static_graph->adj_list))));

       // TODO: define astar_goal_visitor and distance_heuristic

  } catch(found_goal fg) { // found a path to the goal
    std::stack<vg_vertex> stack;
    for(vg_vertex v = goal;; v = p[v]) {
      stack.push( v );
      if(p[v] == v)
        break;
    }

    while( !stack.empty( ) ){
        shortest_path.push_back( stack.top() );
        stack.pop( );
    }

//    std::cout << "Shortest path from " << init << " to " << goal << ": ";
//    std::vector<vg_vertex>::iterator spi = shortest_path.begin();
//    std::cout << init;
//    for(++spi; spi != shortest_path.end(); ++spi)
//        std::cout << " -> " << *spi;
//        std::cout << std::endl << "Total distance: " << d[goal] << std::endl;

    return d[goal];
  }
  return -1;
}


void mousePassiveMotionFunc( int x,int y ){

}
